
/*
 * File CASCMultiMode.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J8. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J8 only because so is required by the CASC-J8
 * rules, and for no other reason.
 */


#ifndef __CASCMultiMode__
#define __CASCMultiMode__

#include <utility>

#include "Forwards.hpp"

#include "Lib/DHSet.hpp"
#include "Lib/Portability.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "Lib/VString.hpp"

#include "Lib/Sys/SyncPipe.hpp"

#include "Kernel/Problem.hpp"

#include "Shell/Property.hpp"
#include "Shell/SineUtils.hpp"

namespace CASC {

using namespace std;
using namespace Lib;
using namespace Kernel;



#if COMPILER_MSVC

class CASCMultiMode
{
public:
  static bool perform() { USER_ERROR("multi-core casc  mode is not supported on Windows"); }
};

#else

class CASCMultiMode
{
    enum {
        SEM_LOCK = 0,
        SEM_PRINTED = 1
    };
public:

  CASCMultiMode() : _syncSemaphore(2)
{
    
    
    
    
    _syncSemaphore.set(SEM_LOCK,1);    
    _syncSemaphore.set(SEM_PRINTED,0); 
}

  static bool perform();
private:
  static ostream& lineOutput();
  static ostream& coutLineOutput();

  typedef Set<vstring> StrategySet;
  typedef Stack<vstring> Schedule;

  bool searchForProof();

  bool runSchedule(Schedule&,StrategySet& remember,bool fallback,int terminationTime);
  unsigned getSliceTime(vstring sliceCode,vstring& chopped);

  bool performStrategy(Shell::Property* property);
  bool waitForChildAndCheckIfProofFound();

  void runSlice(vstring slice, unsigned milliseconds) __attribute__((noreturn));
  void runSlice(Options& strategyOpt) __attribute__((noreturn));


#if VDEBUG
  DHSet<pid_t> childIds;
#endif


 
  ScopedPtr<Problem> prb;

  Semaphore _syncSemaphore; 
};

#endif 

}

#endif 
