
/*
 * File FiniteModel.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J8. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J8 only because so is required by the CASC-J8
 * rules, and for no other reason.
 */


#ifndef __FiniteModel__
#define __FiniteModel__

#include "Lib/Allocator.hpp"
#include "Lib/DHMap.hpp"

#include "Kernel/Unit.hpp"
#include "Kernel/Term.hpp"


namespace FMB {

using namespace Lib;
using namespace Kernel;

class FiniteModel {
 CLASS_NAME(FiniteModel);
 USE_ALLOCATOR(FiniteModel);

public:

 const unsigned _size;
 FiniteModel(unsigned size);

 
 
 void addConstantDefinition(unsigned f, unsigned res);
 void addFunctionDefinition(unsigned f, const DArray<unsigned>& args, unsigned res); 
 
 void addPropositionalDefinition(unsigned f, bool res);
 void addPredicateDefinition(unsigned f, const DArray<unsigned>& args, bool res); 

 bool isPartial();

 bool evaluate(Unit* unit);
 unsigned evaluateGroundTerm(Term* term);
 bool evaluateGroundLiteral(Literal* literal);

 vstring toString();

private:

 Formula* partialEvaluate(Formula* formula);
 
 bool evaluate(Formula* formula,unsigned depth=0);

 
 
 bool _isPartial;
 DHMap<unsigned,unsigned> _functionCoverage;
 DHMap<unsigned,unsigned> _predicateCoverage;

 DArray<unsigned> f_offsets;
 DArray<unsigned> p_offsets;
 DArray<unsigned> f_interpretation;
 DArray<unsigned> p_interpretation; 

 DHMap<unsigned,Term*> _domainConstants;
 DHMap<Term*,unsigned> _domainConstantsRev;
public:
 Term* getDomainConstant(unsigned c)
 {
   Term* t;
   if(_domainConstants.find(c,t)) return t;
   vstring name = "domainConstant";
   unsigned f = env.signature->addFreshFunction(0,name.c_str()); 
   t = Term::createConstant(f);
   _domainConstants.insert(c,t);
   _domainConstantsRev.insert(t,c);

   return t;
 }
 unsigned getDomainConstant(Term* t)
 {
   unsigned c;
   if(_domainConstantsRev.find(t,c)) return c;
   USER_ERROR("Evaluated to "+t->toString()+" when expected a domain constant, probably a partial model");
 }
 bool isDomainConstant(Term* t)
 {
   return _domainConstantsRev.find(t);
 }

};


} 
#endif
